@@ -33,7 +33,7 @@
  * CopyRight (c) 2003, Robert Collins <robertc@squid-cache.org>
  */
 
-#include "squid.h"
+#include "squid-old.h"
 
 #include <sys/ipc.h>
 #include <sys/msg.h>
