@@ -33,7 +33,7 @@
  *
  */
 
-#include "squid.h"
+#include "squid-old.h"
 #include "StoreMetaUnpacker.h"
 #include "StoreMeta.h"
 
