@@ -50,11 +50,11 @@
  * modules from creating several versions of the same function simultaneously.
  */
 
-#include "config.h"
+#include "squid.h"
 
 #if USE_DEVPOLL
 
-#include "squid.h"
+#include "squid-old.h"
 #include "comm/Loops.h"
 #include "fde.h"
 #include "mgr/Registration.h"
