@@ -33,7 +33,7 @@
  *
  */
 
-#include "squid.h"
+#include "squid-old.h"
 #if USE_CACHE_DIGESTS
 
 #include "event.h"
