@@ -31,7 +31,7 @@
  *
  */
 
-#include "squid.h"
+#include "squid-old.h"
 
 #ifndef SQUID_TYPEDEFS_H
 #define SQUID_TYPEDEFS_H
@@ -47,9 +47,6 @@ typedef struct {
     size_t kb;
 } kb_t;
 
-struct http_port_list;
-struct https_port_list;
-
 typedef struct _close_handler close_handler;
 
 typedef struct _dread_ctrl dread_ctrl;
